"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClsInterceptor = void 0;
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
const cls_service_manager_1 = require("../cls-service-manager");
const cls_constants_1 = require("../cls.constants");
const cls_options_1 = require("../cls.options");
let ClsInterceptor = class ClsInterceptor {
    constructor(options) {
        this.options = Object.assign(Object.assign({}, new cls_options_1.ClsInterceptorOptions()), options);
    }
    intercept(context, next) {
        const cls = cls_service_manager_1.ClsServiceManager.getClsService();
        const clsStore = this.createOrReuseStore(context, cls);
        return new rxjs_1.Observable((subscriber) => {
            cls.runWith(clsStore, async () => {
                var _a, _b;
                if (this.options.generateId) {
                    const id = await ((_b = (_a = this.options).idGenerator) === null || _b === void 0 ? void 0 : _b.call(_a, context));
                    cls.setIfUndefined(cls_constants_1.CLS_ID, id);
                }
                if (this.options.setup) {
                    await this.options.setup(cls, context);
                }
                if (this.options.initializePlugins) {
                    await cls.initializePlugins();
                }
                if (this.options.resolveProxyProviders) {
                    await cls.resolveProxyProviders();
                }
                try {
                    next.handle()
                        .pipe()
                        .subscribe({
                        next: (res) => subscriber.next(res),
                        error: (err) => subscriber.error(err),
                        complete: () => subscriber.complete(),
                    });
                }
                catch (e) {
                    subscriber.error(e);
                }
            });
        });
    }
    createOrReuseStore(context, cls) {
        let store = (cls.isActive() && cls.get()) || {};
        // NestJS triggers the interceptor for all queries within the same
        // call individually, so each query would be wrapped in a different
        // CLS context.
        // The solution is to store the CLS store in the GQL context and re-use
        // it each time the interceptor is triggered within the same request.
        if (context.getType() == 'graphql') {
            const gqlContext = context.getArgByIndex(2);
            if (!gqlContext.__CLS_STORE__) {
                gqlContext.__CLS_STORE__ = store;
            }
            else {
                store = gqlContext.__CLS_STORE__;
            }
        }
        return store;
    }
};
ClsInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(cls_constants_1.CLS_INTERCEPTOR_OPTIONS)),
    __metadata("design:paramtypes", [Object])
], ClsInterceptor);
exports.ClsInterceptor = ClsInterceptor;
//# sourceMappingURL=cls.interceptor.js.map