"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ClsModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClsModule = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const cls_guard_1 = require("./cls-initializers/cls.guard");
const cls_interceptor_1 = require("./cls-initializers/cls.interceptor");
const cls_middleware_1 = require("./cls-initializers/cls.middleware");
const cls_service_manager_1 = require("./cls-service-manager");
const cls_constants_1 = require("./cls.constants");
const cls_options_1 = require("./cls.options");
const cls_service_1 = require("./cls.service");
const cls_plugin_manager_1 = require("./plugin/cls-plugin-manager");
const proxy_provider_manager_1 = require("./proxy-provider/proxy-provider-manager");
const clsServiceProvider = {
    provide: cls_service_1.ClsService,
    useValue: cls_service_manager_1.ClsServiceManager.getClsService(),
};
const commonProviders = [
    clsServiceProvider,
    proxy_provider_manager_1.ProxyProviderManager.createProxyProviderFromExistingKey(cls_constants_1.CLS_REQ),
    proxy_provider_manager_1.ProxyProviderManager.createProxyProviderFromExistingKey(cls_constants_1.CLS_RES),
];
let ClsModule = ClsModule_1 = class ClsModule {
    constructor(adapterHost, moduleRef) {
        this.adapterHost = adapterHost;
        this.moduleRef = moduleRef;
    }
    configure(consumer) {
        if (!this.isForRootImport())
            return;
        const options = this.moduleRef.get(cls_constants_1.CLS_MIDDLEWARE_OPTIONS);
        const adapter = this.adapterHost.httpAdapter;
        let mountPoint = '*';
        if (adapter.constructor.name === 'FastifyAdapter') {
            mountPoint = '(.*)';
        }
        if (options.mount) {
            ClsModule_1.logger.debug('Mounting ClsMiddleware to ' + mountPoint);
            consumer.apply(cls_middleware_1.ClsMiddleware).forRoutes(mountPoint);
        }
    }
    isForRootImport() {
        // CLS_MODULE_OPTIONS is only available if the module is imported with `forRoot/Async`
        try {
            this.moduleRef.get(cls_constants_1.CLS_MODULE_OPTIONS);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * Configures the CLS module in the root.
     *
     * Provides the `ClsService` for injection.
     */
    static forRoot(options) {
        options = Object.assign(Object.assign({}, new cls_options_1.ClsModuleOptions()), options);
        const { providers, exports } = this.getProviders();
        const proxyProviders = this.createProxyClassProviders(options.proxyProviders);
        return {
            module: ClsModule_1,
            imports: cls_plugin_manager_1.ClsPluginManager.registerPlugins(options.plugins),
            providers: [
                {
                    provide: cls_constants_1.CLS_MODULE_OPTIONS,
                    useValue: options,
                },
                ...providers,
                ...proxyProviders,
            ],
            exports: [...exports, ...proxyProviders.map((p) => p.provide)],
            global: options.global,
        };
    }
    /**
     * Configures the CLS module in the root with asynchronously provided configuration.
     *
     * Provides the `ClsService` for injection.
     */
    static forRootAsync(asyncOptions) {
        var _a;
        const { providers, exports } = this.getProviders();
        const proxyProviders = this.createProxyClassProviders(asyncOptions.proxyProviders);
        return {
            module: ClsModule_1,
            imports: [
                ...((_a = asyncOptions.imports) !== null && _a !== void 0 ? _a : []),
                ...cls_plugin_manager_1.ClsPluginManager.registerPlugins(asyncOptions.plugins),
            ],
            providers: [
                {
                    provide: cls_constants_1.CLS_MODULE_OPTIONS,
                    inject: asyncOptions.inject,
                    useFactory: asyncOptions.useFactory,
                },
                ...providers,
                ...proxyProviders,
            ],
            exports: [...exports, ...proxyProviders.map((p) => p.provide)],
            global: asyncOptions.global,
        };
    }
    static forFeature(...proxyProviderClasses) {
        const proxyProviders = this.createProxyClassProviders(proxyProviderClasses);
        const providers = [...commonProviders];
        return {
            module: ClsModule_1,
            providers: [...providers, ...proxyProviders],
            exports: [...providers, ...proxyProviders.map((p) => p.provide)],
        };
    }
    /**
     * Registers the given Class or Factory proxy providers in the module along with `ClsService`.
     *
     * If used with `global: true`, makes the proxy provider available globally.
     */
    static forFeatureAsync(options) {
        var _a, _b;
        const proxyProvider = proxy_provider_manager_1.ProxyProviderManager.createProxyProvider(options);
        const providers = [
            ...commonProviders,
            ...((_a = options.extraProviders) !== null && _a !== void 0 ? _a : []),
        ];
        return {
            module: ClsModule_1,
            imports: (_b = options.imports) !== null && _b !== void 0 ? _b : [],
            providers: [...providers, proxyProvider],
            exports: [...commonProviders, proxyProvider.provide],
            global: options.global,
        };
    }
    static createProxyClassProviders(proxyProviderClasses) {
        var _a;
        return ((_a = proxyProviderClasses === null || proxyProviderClasses === void 0 ? void 0 : proxyProviderClasses.map((providerClass) => proxy_provider_manager_1.ProxyProviderManager.createProxyProvider({
            useClass: providerClass,
        }))) !== null && _a !== void 0 ? _a : []);
    }
    static getProviders() {
        const providers = [
            ...commonProviders,
            {
                provide: cls_constants_1.CLS_MIDDLEWARE_OPTIONS,
                inject: [cls_constants_1.CLS_MODULE_OPTIONS],
                useFactory: this.clsMiddlewareOptionsFactory,
            },
            {
                provide: cls_constants_1.CLS_GUARD_OPTIONS,
                inject: [cls_constants_1.CLS_MODULE_OPTIONS],
                useFactory: this.clsGuardOptionsFactory,
            },
            {
                provide: cls_constants_1.CLS_INTERCEPTOR_OPTIONS,
                inject: [cls_constants_1.CLS_MODULE_OPTIONS],
                useFactory: this.clsInterceptorOptionsFactory,
            },
        ];
        const enhancerArr = [
            {
                provide: core_1.APP_GUARD,
                inject: [cls_constants_1.CLS_GUARD_OPTIONS],
                useFactory: this.clsGuardFactory,
            },
            {
                provide: core_1.APP_INTERCEPTOR,
                inject: [cls_constants_1.CLS_INTERCEPTOR_OPTIONS],
                useFactory: this.clsInterceptorFactory,
            },
        ];
        return {
            providers: providers.concat(...enhancerArr),
            exports: providers,
        };
    }
    static clsMiddlewareOptionsFactory(options) {
        const clsMiddlewareOptions = Object.assign(Object.assign({}, new cls_options_1.ClsMiddlewareOptions()), options.middleware);
        return clsMiddlewareOptions;
    }
    static clsGuardOptionsFactory(options) {
        const clsGuardOptions = Object.assign(Object.assign({}, new cls_options_1.ClsGuardOptions()), options.guard);
        return clsGuardOptions;
    }
    static clsInterceptorOptionsFactory(options) {
        const clsInterceptorOptions = Object.assign(Object.assign({}, new cls_options_1.ClsInterceptorOptions()), options.interceptor);
        return clsInterceptorOptions;
    }
    static clsGuardFactory(options) {
        if (options.mount) {
            ClsModule_1.logger.debug('ClsGuard will be automatically mounted');
            return new cls_guard_1.ClsGuard(options);
        }
        return {
            canActivate: () => true,
        };
    }
    static clsInterceptorFactory(options) {
        if (options.mount) {
            ClsModule_1.logger.debug('ClsInterceptor will be automatically mounted');
            return new cls_interceptor_1.ClsInterceptor(options);
        }
        return {
            intercept: (_, next) => next.handle(),
        };
    }
};
ClsModule.logger = new common_1.Logger(ClsModule_1.name);
ClsModule = ClsModule_1 = __decorate([
    (0, common_1.Module)({
        providers: [...commonProviders],
        exports: [...commonProviders],
    }),
    __metadata("design:paramtypes", [core_1.HttpAdapterHost,
        core_1.ModuleRef])
], ClsModule);
exports.ClsModule = ClsModule;
//# sourceMappingURL=cls.module.js.map