"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProxyProviderNotDecoratedException = exports.UnknownProxyDependenciesException = void 0;
class UnknownProxyDependenciesException extends Error {
    constructor() {
        super(...arguments);
        this.name = UnknownProxyDependenciesException.name;
    }
    static create(error, Provider) {
        var _a, _b;
        const expectedParams = Reflect.getMetadata('design:paramtypes', Provider);
        const foundParams = this.extractDependencyParams(error);
        const notFoundParamIndex = foundParams.findIndex((it) => it == '?');
        let notFoundParamName = (_a = expectedParams[notFoundParamIndex]) === null || _a === void 0 ? void 0 : _a.name;
        if (!notFoundParamName) {
            notFoundParamName = (_b = Reflect.getMetadata('self:paramtypes', Provider)) === null || _b === void 0 ? void 0 : _b.find((param) => (param === null || param === void 0 ? void 0 : param.index) == notFoundParamIndex).param.toString();
        }
        const message = this.composeMessage(Provider.name, foundParams.join(', '), notFoundParamName, notFoundParamIndex);
        return new UnknownProxyDependenciesException(message);
    }
    static extractDependencyParams(error) {
        var _a, _b;
        // matches the parameters from NestJS's error message:
        // e.g: "Nest can't resolve dependencies of the Something (Cats, ?). [...]"
        // returns ['Cats', '?']
        return (_b = (_a = error.message.match(/\w+ \((.*?)\)./)) === null || _a === void 0 ? void 0 : _a[1].split(', ')) !== null && _b !== void 0 ? _b : [];
    }
    static composeMessage(providerName, foundParamsString, notFoundParamName, notFoundIndex) {
        return (`Cannot create Proxy provider ${providerName} (${foundParamsString}). ` +
            `The argument ${notFoundParamName} at index [${notFoundIndex}] was not found in the ClsModule Context.\n\n` +
            `Potential solutions:\n` +
            `- If ${notFoundParamName} is a provider from a separate module, make sure to import the module in "ClsModule.forFeatureAsync()" registration`);
    }
}
exports.UnknownProxyDependenciesException = UnknownProxyDependenciesException;
class ProxyProviderNotDecoratedException extends Error {
    constructor() {
        super(...arguments);
        this.name = ProxyProviderNotDecoratedException.name;
    }
    static create(Provider) {
        const message = `Cannot create a Proxy provider for ${Provider.name}. The class must be explicitly decorated with the @InjectableProxy() decorator to distinguish it from a regular provider.`;
        return new ProxyProviderNotDecoratedException(message);
    }
}
exports.ProxyProviderNotDecoratedException = ProxyProviderNotDecoratedException;
//# sourceMappingURL=proxy-provider.exceptions.js.map